@@ -52,6 +52,11 @@
 #include <clutter-gtk/clutter-gtk.h>
 #include <clutter/x11/clutter-x11.h>
 
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <GL/gl.h>
+#include <GL/glx.h>
+
 #include <libnotify/notify.h>
 #include <launcher/launcher.h>
 
@@ -160,6 +165,96 @@ on_session_quit (void)
   gtk_main_quit ();
 }
 
+/* take the optimistic choice: only return FALSE when we are sure it's FALSE */
+static gboolean
+rendering_available (void)
+{
+  char *displayName = NULL;
+  gchar *renderer = NULL;
+  Display *dpy;
+  int scrnum = 0;
+  Bool allowDirect = True;
+  Window win;
+  XSetWindowAttributes attr;
+  unsigned long mask;
+  Window root;
+  GLXContext ctx = NULL;
+  XVisualInfo *visinfo;
+  int width = 100, height = 100;
+  int attribSingle[] = {
+     GLX_RGBA,
+     GLX_RED_SIZE, 1,
+     GLX_GREEN_SIZE, 1,
+     GLX_BLUE_SIZE, 1,
+     None };
+  int attribDouble[] = {
+     GLX_RGBA,
+     GLX_RED_SIZE, 1,
+     GLX_GREEN_SIZE, 1,
+     GLX_BLUE_SIZE, 1,
+     GLX_DOUBLEBUFFER,
+     None };
+
+  dpy = XOpenDisplay(displayName); 
+  if (!dpy)
+   {
+    fprintf(stderr, "Error: unable to open display %s\n", XDisplayName(displayName));
+    return TRUE;
+   }
+  root = RootWindow(dpy, scrnum);
+  visinfo = glXChooseVisual(dpy, scrnum, attribSingle);
+  if (!visinfo)
+     visinfo = glXChooseVisual(dpy, scrnum, attribDouble);
+  if (visinfo)
+     ctx = glXCreateContext( dpy, visinfo, NULL, allowDirect );
+  if (!visinfo)
+   {
+      fprintf(stderr, "Error: couldn't find RGB GLX visual or fbconfig\n");
+      return TRUE;
+    }
+   if (!ctx)
+    {
+      fprintf(stderr, "Error: glXCreateContext failed\n");
+      XFree(visinfo);
+      return TRUE;
+    }
+
+   attr.background_pixel = 0;
+   attr.border_pixel = 0;
+   attr.colormap = XCreateColormap(dpy, root, visinfo->visual, AllocNone);
+   attr.event_mask = StructureNotifyMask | ExposureMask;
+   mask = CWBackPixel | CWBorderPixel | CWColormap | CWEventMask;
+
+   win = XCreateWindow(dpy, root, 0, 0, width, height,
+                       0, visinfo->depth, InputOutput,
+                       visinfo->visual, mask, &attr);
+
+   if (glXMakeCurrent(dpy, win, ctx))
+    {
+      const char *glRenderer = (const char *) glGetString(GL_RENDERER);
+      renderer = g_ascii_strdown (glRenderer, -1);
+      g_debug ("OpenGL renderer string: %s\n", glRenderer);
+    }
+
+    if (renderer && strstr(renderer, "software"))
+       return FALSE;
+    return TRUE;
+}
+
+static gint
+fallback_to_efl (void)
+{
+  GError *error = NULL;
+  
+  if (!g_spawn_command_line_async ("/usr/bin/netbook-launcher-efl", &error))
+    {
+      g_print ("Error in netbook-launcher-efl: %s\n", error->message);
+      g_error_free (error);
+      return EXIT_FAILURE;
+    }
+  return 0;
+}
+
 gint
 main (gint argc, gchar *argv[])
 {
@@ -184,6 +279,13 @@ main (gint argc, gchar *argv[])
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  if (!rendering_available ())
+    {
+      g_print ("No rendering avaible for netbook-launcher, try to run netbook-launcher-efl\n");
+      
+      return fallback_to_efl ();
+    }
+
   if (!notify_init ("Netbook Launcher"))
     return EXIT_FAILURE;
 
@@ -286,6 +388,12 @@ main (gint argc, gchar *argv[])
   /* Create the nl window */
   window = nl_window_get_default ();
 
+  if (!nl_window_is_valid (window))
+    {
+      g_print ("Error creating clutter launcher.  Trying to run netbook-launcher-efl\n");
+      return fallback_to_efl ();
+    }
+
   plugin_manager = nl_plugin_manager_new (NL_SHELL (window));
   folders_source = nl_folders_source_new (NL_SHELL (window));
   volumes_source = nl_volumes_source_new (NL_SHELL (window));
