@@ -161,6 +161,7 @@ nl_window_constructed (GObject *object)
   NlConfig        *cfg = nl_config_get_default ();
   ClutterActor    *vbox, *arrow, *bg;
   extern gboolean  windowed;
+  GtkTextDirection direction;
 
   priv->client = gconf_client_get_default ();
 
@@ -191,9 +192,15 @@ nl_window_constructed (GObject *object)
   priv->stage = gtk_clutter_embed_get_stage
                 (GTK_CLUTTER_EMBED (priv->gtkclutter));
 
+  // Given sufficiently poor OpenGL drivers getting the stage
+  // can fail.  In this case, there's a good chance that further
+  // OpenGL calls will fail, sometimes spectacularly.
+  // See LP: #467474 for an example.
+  g_return_if_fail (CLUTTER_IS_STAGE (priv->stage));
+
   /* Set NlWidget's direction */
-  ctk_actor_set_default_direction (
-    gtk_widget_get_default_direction ());
+  direction = gtk_widget_get_default_direction ();
+  ctk_actor_set_default_direction (direction);
 
   /* Background first */
   priv->background = nl_background_new ();
@@ -242,11 +249,19 @@ nl_window_constructed (GObject *object)
   /* Sidebar */
   if (!CLUTTER_IS_ACTOR (normal_texture))
     {
-      normal_texture = clutter_texture_new_from_file
-                       (PKGDATADIR"/sidebar_bg.png", NULL);
+      if (direction == GTK_TEXT_DIR_RTL)
+        normal_texture = clutter_texture_new_from_file
+                         (PKGDATADIR"/sidebar_bg_rtl.png", NULL);
+      else
+        normal_texture = clutter_texture_new_from_file
+                         (PKGDATADIR"/sidebar_bg.png", NULL);
     }
-  bg = nl_texture_frame_new (CLUTTER_TEXTURE (normal_texture),
-                             0, 30, 0, 0);
+  if (direction == GTK_TEXT_DIR_RTL)
+      bg = nl_texture_frame_new (CLUTTER_TEXTURE (normal_texture),
+                                 0, 0, 0, 30);
+  else
+      bg = nl_texture_frame_new (CLUTTER_TEXTURE (normal_texture),
+                                 0, 30, 0, 0);
   ctk_actor_set_background (CTK_ACTOR (priv->sidebar_scroll), bg);
   ctk_scroll_view_set_bar_type (CTK_SCROLL_VIEW (priv->sidebar_scroll),
                                 CTK_SCROLLBAR_HIDDEN);
@@ -361,6 +376,14 @@ nl_window_get_default (void)
   return window;
 }
 
+gboolean
+nl_window_is_valid (GtkWidget *window)
+{
+  // This is not all of the state required to declare a window valid, but
+  // it's enough to LP: #467474
+  return CLUTTER_IS_STAGE (NL_WINDOW (window)->priv->stage);
+}
+
 /*
  * Private methods
  */
